@@ -1,9 +1,12 @@
 
 package Coro::ProcessPool::Pipeline;
 # ABSTRACT: A producer/consumer pipeline for Coro::ProcessPool
-$Coro::ProcessPool::Pipeline::VERSION = '0.28';
+$Coro::ProcessPool::Pipeline::VERSION = '0.29';
+use strict;
+use warnings;
 use Carp;
 use Coro;
+use Try::Catch;
 
 sub new {
   my ($class, %param) = @_;
@@ -41,18 +44,24 @@ sub queue {
 
   async_pool {
     my ($self, $deferred) = @_;
-    my $result = eval { $deferred->recv };
 
-    $self->{complete}->put([$result, $@]);
-    --$self->{num_pending};
-
-    if ($self->{num_pending} == 0) {
-      if ($self->{shutting_down} || $self->{auto_shutdown}) {
-        $self->{complete}->shutdown;
-        $self->{is_shutdown} = 1;
-        $self->{shutting_down} = 0;
-      }
+    try {
+      my $result = $deferred->recv;
+      $self->{complete}->put([$result, undef]);
     }
+    catch {
+      $self->{complete}->put([undef, $_]);
+    }
+    finally {
+      if (--$self->{num_pending} == 0) {
+        if ($self->{shutting_down} || $self->{auto_shutdown}) {
+          $self->{complete}->shutdown;
+          $self->{is_shutdown} = 1;
+          $self->{shutting_down} = 0;
+        }
+      }
+    };
+
   } $self, $deferred;
 
   ++$self->{num_pending};
@@ -78,7 +87,7 @@ Coro::ProcessPool::Pipeline - A producer/consumer pipeline for Coro::ProcessPool
 
 =head1 VERSION
 
-version 0.28
+version 0.29
 
 =head1 SYNOPSIS
 
@@ -105,10 +114,6 @@ version 0.28
 Provides an iterative mechanism for feeding tasks into the process pool and
 collecting results. A pool may have multiple pipelines.
 
-=head1 NAME
-
-Coro::ProcessPool::Pipeline
-
 =head1 ATTRIBUTES
 
 =head2 pool (required)
